<?php

namespace App\Repository;

use App\Dto\SnipFilterRequest;
use App\Entity\Snip;
use App\Entity\User;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Component\Security\Core\User\UserInterface;

/**
 * @extends ServiceEntityRepository<Snip>
 *
 * @method Snip|null find($id, $lockMode = null, $lockVersion = null)
 * @method Snip|null findOneBy(array $criteria, array $orderBy = null)
 * @method Snip[]    findAll()
 * @method Snip[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class SnipRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Snip::class);
    }

    public function save(Snip $entity, bool $flush = true): void
    {
        $this->getEntityManager()->persist($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function remove(Snip $entity, bool $flush = true): void
    {
        $this->getEntityManager()->remove($entity);

        if ($flush) {
            $this->getEntityManager()->flush();
        }
    }

    public function findByRequest(UserInterface $user, SnipFilterRequest $request): array
    {
        $qb = $this
            ->createQueryBuilder('s')
            ->where('s.createdBy = :user')
            ->setParameter('user', $user)
            ->orderBy('s.name', 'ASC')
        ;

        $qb->andWhere('s.visible = :visible')
           ->setParameter('visible', $request->onlyVisible)
        ;

        return $qb->getQuery()->getResult();
    }

    public function findPublic(?User $user = null): array
    {
        $qb = $this
            ->createQueryBuilder('s')
            ->where('s.public = true')
            ->andWhere('s.visible = true')
            ->orderBy('s.createdAt', 'DESC')
        ;

        if ($user) {
            $qb->andWhere('s.createdBy != :user')
               ->setParameter('user', $user)
            ;
        }

        return $qb->getQuery()->getResult();
    }
}
